@@ -5,6 +5,10 @@
 #if defined(__GLASGOW_HASKELL__) && __GLASGOW_HASKELL__ >= 706
 {-# LANGUAGE PolyKinds #-}
 #endif
+#if defined(__GLASGOW_HASKELL__) && __GLASGOW_HASKELL__ >= 707
+{-# LANGUAGE StandaloneDeriving #-}
+#endif
+{-# OPTIONS_GHC -fno-warn-orphans #-}
 ----------------------------------------------------------------------------
 -- |
 -- Module     : Data.Proxy
@@ -38,7 +42,11 @@ import GHC.Arr (unsafeIndex, unsafeRangeSize)
 import Data.Data
 #endif
 
+#if defined(__GLASGOW_HASKELL__) && __GLASGOW_HASKELL__ >= 707
+deriving instance Typeable Proxy
+#else
 data Proxy s = Proxy
+#endif
 
 instance Eq (Proxy s) where
   _ == _ = True
@@ -53,7 +61,7 @@ instance Read (Proxy s) where
   readsPrec d = readParen (d > 10) (\r -> [(Proxy, s) | ("Proxy",s) <- lex r ])
 
 #ifdef __GLASGOW_HASKELL__
-
+#if __GLASGOW_HASKELL__ < 707
 instance Typeable1 Proxy where
   typeOf1 _ = mkTyConApp proxyTyCon []
 
@@ -64,6 +72,7 @@ proxyTyCon = mkTyCon "Data.Proxy.Proxy"
 proxyTyCon = mkTyCon3 "tagged" "Data.Proxy" "Proxy"
 #endif
 {-# NOINLINE proxyTyCon #-}
+#endif
 
 instance Data s => Data (Proxy s) where
   gfoldl _ z _ = z Proxy
