@@ -26,6 +26,7 @@ module Data.Tagged
     , tagSelf
     , untagSelf
     , asTaggedTypeOf
+    , witness
     ) where
 
 import Control.Applicative ((<$>), liftA2, Applicative(..))
@@ -47,9 +48,13 @@ import Data.Monoid
 -- argument, because the newtype is \"free\"
 newtype Tagged s b = Tagged { unTagged :: b } deriving
   ( Eq, Ord, Ix, Bounded
+#if __GLASGOW_HASKELL__ >= 707
+  , Typeable
+#endif
   )
 
 #ifdef __GLASGOW_HASKELL__
+#if __GLASGOW_HASKELL__ < 707
 instance Typeable2 Tagged where
   typeOf2 _ = mkTyConApp taggedTyCon []
 
@@ -60,6 +65,8 @@ taggedTyCon = mkTyCon "Data.Tagged.Tagged"
 taggedTyCon = mkTyCon3 "tagged" "Data.Tagged" "Tagged"
 #endif
 
+#endif
+
 instance (Data s, Data b) => Data (Tagged s b) where
   gfoldl f z (Tagged b) = z Tagged `f` b
   toConstr _ = taggedConstr
@@ -244,6 +251,10 @@ asTaggedTypeOf :: s -> Tagged s b -> s
 asTaggedTypeOf = const
 {-# INLINE asTaggedTypeOf #-}
 
+witness :: Tagged a b -> a -> b
+witness (Tagged b) _ = b
+{-# INLINE witness #-}
+
 -- | 'untagSelf' is a type-restricted version of 'untag'.
 untagSelf :: Tagged a a -> a
 untagSelf (Tagged x) = x
